//Released under     GNU LESSER GENERAL PUBLIC LICENSE    http://www.gnu.org/licenses/lgpl.html

var mathml = function() {


//@@@@@@@      USER DEFINED AREA       @@@@@@@@@@@//

var config = {

	tidy_on_load: true, // Set to false if you don't want the mathml to be tidied straight away, then make a call to    mathml.tidy(node)  or  mathml.init()  when you are ready	
	
	use_images_for_large_operators: true, // Large operators look better (more TEX like) when this is true, and the images are all the same size.
	                                      // If fonts are used then the sizes vary, the vertical alignment is poorer, but processing may be slightly quicker due to not loading images.

	stretch_single_parentheses: false,   //if true, will stretch parentheses such as <mo>(</mo>.....<mo>)</mo>. If false, will parentheses require a      stretchy="true"      attribute to be stretched

	use_images_for_large_unfenced_parentheses: true,  //if false then large fonts are used. recommend you leave this as true as otherwise parentheses are too fat for large elements

	auto_horizontal_stretch: true,    //it's better to leave this false and attach     stretchy="true"    to those elements you wish to horizontally stretch

	flushed_msubsup: true,  //If false then msubsup elements might not line up flushed (especially when the subscript is longer than one character), but you will save time on processing
	ie_flushed_msubsup: true, //Same variable but IE only.
	
	show_mathml_on_mousedown: false, //if true, when the user clicks (or even starts to drag over) a mathml tag, the mathml will appear in a pop up box. //TODO: get working for IE
	magnify_on_mousedown: false, //mousedown on an equation to have it appear large. The appearance when large is determined in the css file by     math[magnify="true"]     //TODO: get working for IE

	ie7_warning: 'The mathematics on this page looks bad when displayed using Internet Explorer 7.\n\nYou should use a standards compliant browser such as Firefox, Safari or Chrome.', //Set this to an empty string if you don't want a dialog box to appear for IE7

	debug: false // For testing purposes, if you want to know how long various functions are taking to execute set this to true. Make sure you set it to false after testing.
};



//The following <mo> symbols won't be given any padding. You can also use padded="false" directly on the element
var nopadding_array = [
	' ',
	'\u00A0', //non-breaking space
	'(',
	')',
	'[',
	']',
	'{',
	'}',
	'|',
	'\u2223', //same as above
	'||',
	'\u2225', //same as above
	'.',
	'. ',
	',',
	', ',
	';',
	'; ',
	':',
	"'",
	'"',
	'\u22C5', //centered dot
	'\u2219', //  "          "
	'\\',
	'/',
	'!',
	'?',
	'%',
	'$',
	'&',
	'#',
	'@',
	'\u20AC', //euro
	'\u27E8', //left angular
	'\u2329', //  "          "
	'\u27E9', //right angular
	'\u232A', //  "          "
	'\u23B0', //left moustache
	'\u23B1', //right moustache
	'\u2200', //for all symbol
	'\u2203', //exists symbol
	'\u2205', //empty set
	'\u2135', //aleph
	'\u2202', //partial d
	'\u00AC', //not symbol
	'\u2207', //nabla
	'\u22A5', //perpendicular
	'\u22A4', //above symbol other way up
	'\u0127', //h with a bar
	'\u2373', //i without a dot
	'\u006A', //j without a dot
	'\u2113', //lower case script L
	'\u2118', //
	'\u211C', //
	'\u2111', //
	'\u2220', //
	'\u25B3', //
	'\u266D', //flat
	'\u266E', //
	'\u266F', //sharp
	'\u2026', //lower 3 dots
	'\u22EF', //centered 3 dots
	'\u22EE', //vertical 3 dots
	'\u22F1', //diagonal 3 dots
	'\u2102', //complex numbers
	'\u210D', //H with same style
	'\u2115', //natural numbers
	'\u2119', //P with same style
	'\u211A', //rationals
	'\u211D', //reals
	'\u2124', //integers
	'\u2204', //not exists
	'\u2205',
	'\u2136',
	'\u2137',
	'\u2138',
	'\u00F0',
	'\u210F',
	'\u2127',
	'\u2201',
	'\u2035',
	'\u2220',
	'\u2221',
	'\u2221',
	'\u25B5',
	'\u25B4',
	'\u25BE',
	'\u25BF',
	'\u25A1',

	'\u2000', // Spaces
	'\u2001',
	'\u2002',
	'\u2003',
	'\u2004',
	'\u2005',
	'\u2006',
	'\u2007',
	'\u2008',
	'\u2009',
	'\u200A',
	'\u200B',
	'\u205F' //no comma on last element!
];


var end_of_proof_rep = '&#160;<span style="background-color:black; line-height:1em; font-size:0.9em;">&#160;&#160;&#160;</span>';
var contradiction_rep = '&#160;<img src="MathHTML/Misc/contradiction.png" style="font-size:1em; line-height:1em; height:0.95em;"></img>';

var left_angle_rep = '<img src="MathHTML/Misc/langle.png" style="font-size:1em; line-height:1em; height:0.95em; vertical-align:middle;"></img>';
var right_angle_rep = '<img src="MathHTML/Misc/rangle.png" style="font-size:1em; line-height:1em; height:0.95em; vertical-align:middle;"></img>';

function CharacterReplacement(cha) //Certain (obscure) characters are not displayed on all browsers/operating sytems. This function replaces them with ones that are recognized.
{
	//return '';     //Uncomment this line if you don't want to make any of the replacements below

	switch(cha)
	{
		case '*': return '\u2217'; //default asterix too high
		
		case '\u2329': return left_angle_rep;
		case '\u27E8': return left_angle_rep;
		case '\u232A': return right_angle_rep;
		case '\u27E9': return right_angle_rep;
		
		case '\u25AA': return end_of_proof_rep;
		
		case '\u203B': return contradiction_rep;
		case '\u22C7': return contradiction_rep;
		
		case '\u2A7D': return '\u2264';
		case '\u2A95': return '\u2264';
		case '\u2A7E': return '\u2265';
		case '\u2A96': return '\u2265';
		
		case '\u2A85': return '\u2272';
		case '\u2AB7': return '\u2272';
		case '\u2A86': return '\u2273';
		case '\u2AB8': return '\u2273';
		
		case '\u2A89': return '\u22E8';
		case '\u2AB9': return '\u22E8';
		case '\u2A8A': return '\u22E9';
		case '\u2ABA': return '\u22E9';
		
		case '\u2A8B': return '\u22DA';
		case '\u2A8C': return '\u22DB';
		
		case '\u2AC5': return '\u2286';
		case '\u2AC6': return '\u2287';
		
		case '\u2ACB': return '\u228A';
		case '\u2ACC': return '\u228B';
		
		case '\u2AB5': return '\u2268';
		case '\u2AB6': return '\u2269';
		
		case '\u2373': return '\u03B9';
		
		case '\u2AB7': return '\u227E';
		case '\u2AB8': return '\u227F';
		
		case '\u2A89': return '\u22E6';
		case '\u2A8A': return '\u22E7';

		case '\u29EB': return '\u2666';
		
		case '\u205F': return '<span style="display:inline-block;width:0.22em;"></span>';
		
		default: return '';
	}
}

function LargeOperatorCode(cha) //relevant when  use_images_for_large_operators = true
{
	switch(cha)
	{
		case '\u2211': return 'summation.png';
		case '\u222B': return 'integral.png';
		case '\u220F': return 'product.png';
		case '\u2210': return 'coproduct.png';
		case '\u2229': return 'intersection.png';
		case '\u222A': return 'union.png';
		case '\u2294': return 'squareunion.png';
		case '\u22C1': return 'vee.png';
		case '\u22C0': return 'wedge.png';
		case '\u2295': return 'oplus.png';
		case '\u2297': return 'otimes.png';
		case '\u2299': return 'odot.png';
		case '\u228E': return 'unionplus.png';
		case '\u222E': return 'circleintegral.png';
		case '\u222C': return 'doubleintegral.png';
		case '\u222D': return 'tripleintegral.png';
		case '\u222F': return 'doublecircleintegral.png';
		case '\u2230': return 'triplecircleintegral.png';
		case '\u2231': return 'halfclockwiseintegral.png';
		case '\u2232': return 'clockwiseintegral.png';
		case '\u2233': return 'anticlockwiseintegral.png';
		
		default: return '';
	}
}

function IsIntegral(cha) 
{
	switch(cha)
	{
		case '\u222B': return true; //standard integral
		case '\u222C': return true; //double
		case '\u222D': return true; //triple
		case '\u222E': return true; //contour
		case '\u222F': return true; //surface
		case '\u2230': return true; //volume
		case '\u2231': return true; //clockwise
		case '\u2232': return true; //clockwise contour
		case '\u2233': return true; //anti-clockwise contour
		
		default: return false;
	}
}

function ParenthesisCode(cha) //images are only invoked when they need to be made larger than default text size.
{
	switch(cha)
	{
		case '(':      return 'leftparen.png';
		case ')':      return 'rightparen.png';
		
		case '[':      return 'leftbracket.png';
		case ']':      return 'rightbracket.png';
		
		case '{':      return 'leftbrace.png';
		case '}':      return 'rightbrace.png';
		
		case '|':      return 'pipe.png';
		case '\u2223': return 'pipe.png';
		
		case '||':     return 'doublepipe.png';
		case '\u2225': return 'doublepipe.png';
		
		case '\u2329': return 'leftangular.png'; 
		case '\u27E8': return 'leftangular.png';
		case '\u232A': return 'rightangular.png'; 
		case '\u27E9': return 'rightangular.png';
		
		case '\u230A': return 'leftfloor.png';
		case '\u230B': return 'rightfloor.png';
		
		case '\u2308': return 'leftceiling.png';
		case '\u2309': return 'rightceiling.png';
		
		case '\u2191': return 'uparrow.png';
		case '\u2193': return 'downarrow.png';
		case '\u2195': return 'updownarrow.png';
		
		case '\u21D1': return 'uparrow2.png';
		case '\u21D3': return 'downarrow2.png';
		case '\u21D5': return 'updownarrow2.png';
		
		case '/': return 'forwardslash.png';
		case '\\': return 'backslash.png';
		
		default: return '';
	}
}

function StretchyCode(cha)
{
	switch(cha)
	{
		case '\u2190': return 'leftarrow.png';
		case '\u2192': return 'rightarrow.png';
		case '\u2194': return 'leftrightarrow.png';
		case '\uFE37': return 'overbrace.png';
		case '\uFE38': return 'underbrace.png';
		
		case '^':      return 'widehat.png';
		case '\u02C6': return 'widehat.png';
		
		case '-':      return 'bar.png';
		case '_':      return 'bar.png';
		case '\u00AF': return 'bar.png';

		default: return '';
	}
}

function AccentCode(cha) //Only relevant when   browser_supports_border_image.  
{
	switch(cha)
	{
		case '\u02CB': return 'grave';
		case '\u02C7': return 'check';
		case '\u02D8': return 'breve';
		
		case '\u2192': return 'vec';
		case '\u20D7': return 'vec';

		case '\u02B9': return 'acute';
		case '\u02CA': return 'acute';
		
		case '~':      return 'tilde';
		case '\u02DC': return 'tilde';
		
		case '^':      return 'hat';
		case '\u02C6': return 'hat';
		
		case '.':      return 'dot';
		case '\u02D9': return 'dot';
		case '\u22C5': return 'dot';
		case '\u2219': return 'dot';
		
		case '-':      return 'bar';
		case '_':      return 'bar';
		case '\u00AF': return 'bar';
		case '\u02C9': return 'bar';
		
		case '..':           return 'ddot';
		case '\u00A8':       return 'ddot';
		case '\u22C5\u22C5': return 'ddot';
		case '\u2219\u2219': return 'ddot';

		default: return '';
	}
	
}
function AccentUnderCode(cha) //Only relevant when   browser_supports_border_image.  
{
	switch(cha)
	{
		case '\u2192': return 'undervec';
		case '\u20D7': return 'undervec';
		
		case '-':      return 'underbar';
		case '_':      return 'underbar';
		case '\u00AF': return 'underbar';
		case '\u02C9': return 'underbar';
		
		default: return '';
	}
}

//@@@@@@@      END OF USER DEFINED AREA       @@@@@@@@@@@//




/*  Browser Sniff  */
var ua = navigator.userAgent.toLowerCase();

var ie = (document.getElementById && document.all); var ie6 = false; var ie7 = false; var ie8 = false;
var apple = (ua.indexOf('applewebkit') != -1 ? 1 : 0); //Safari and Google Chrome
var opera = (ua.indexOf('opera') != -1 ? 1 : 0);
var chrome = (ua.indexOf('chrome') != -1 ? 1 : 0);
var firefox = (ua.indexOf('firefox') != -1 ? 1 : 0); 
var safari = false; if (apple && !chrome) {safari = true;}

var browser_supports_border_image = true;
if (opera)
{
	//version 10.5x and above supports border image
	if (ua.indexOf('version/9') != -1 || ua.indexOf('version/8') != -1 || ua.indexOf('version/7') != -1)
	{
		browser_supports_border_image = false;
	}
	else if (ua.indexOf('version/10.0') != -1 || ua.indexOf('version/10.1') != -1 || ua.indexOf('version/10.2') != -1 || ua.indexOf('version/10.3') != -1 || ua.indexOf('version/10.4') != -1)
	{
		browser_supports_border_image = false;
	}
}
if (firefox)
{
	if (ua.indexOf('firefox/2') != -1)
	{
		browser_supports_border_image = false;
	}
}
if (ie)
{
	if (ua.indexOf('msie 9') != -1)
	{
		ie = false; //Hope that IE9 is standards compliant and we don't need any of the hacks 
	}
	else
	{
		if (ua.indexOf('msie 6') != -1) {ie7 = true;}
		if (ua.indexOf('msie 7') != -1) {ie7 = true;}
		if (ua.indexOf('msie 8') != -1) {ie8 = true;}
		browser_supports_border_image = false;
		
		if (ie7)
		{
			if (config.ie7_warning && config.ie7_warning != '') {alert(config.ie7_warning);}
		}
	}
}
if (safari)
{
	if (ua.indexOf('version/2') != -1) {browser_supports_border_image = false;}
	//alert(browser_supports_border_image);
} 



var debug_on = config.debug;
var time_str = '';
var last_time = -1;
var last_sec = -1;
var total_time = 0;

function SetTime(func)
{
	if (!debug_on) {return;}
	
	var date = new Date();
	var time = date.getMilliseconds();
	var sec = date.getSeconds();
	
	var diff = time - last_time;
	diff += (sec - last_sec) * 1000;
	
	if (func == 'START') {diff = 0;}
	
	total_time += diff;
	
	if (func == 'FINISH')
	{
		time_str += 'TOTAL:  ' + total_time + '  \n';
	}
	else
	{
		time_str += func + ':  ' + diff + '  \n';
	}

	last_time = time;
	last_sec = sec;
}

var alerted = false;
function AlertOnce(str)
{
	if (alerted) {return;}
	alert(str);
	alerted = true;
}


/* Some general functions */
function D(id)
{
	return document.getElementById(id);
}
function Trim(str)
{
	if (!str) {return '';}
	return str.replace(/^\s+|\s+$/g, '');
}
function LeftTrim(str)
{
	if (!str) {return '';}
	return str.replace(/^\s+/, '');
}
function RightTrim(str)
{
	if (!str) {return '';}
	return str.replace(/\s+$/, '');
}
function IsUppercase(str)
{
	if (str == str.toUpperCase()) {return true;}
	return false;
}
function IsLowercase(str)
{
	if (str == str.toLowerCase()) {return true;}
	return false;
}
function IsTallLetter(str)
{
	if (IsUppercase(str)) {return true;}
	
	if (str == 'b' || str == 'd' || str == 'f' || str == 'h' || str == 'i' || str == 'j' || str == 'k' || str == 'l' || str == 't') {return true;}
	
	//tall greek
	if (str == '\u03B2' || str == '\u03B4' || str == '\u03B6' || str == '\u03B8' || str == '\u03D1' || str == '\u03BB' || str == '\u03BE' || str == '\u03C6' || str == '\u03C8') {return true;}
	
	return false;
}
function IsSkinnyLetter(str)
{
	if (str == 'i' || str == 'j' || str == 'l' || str == 't' || str == 'I' || str == '\u03B9') {return true;}
	return false;
}


/* Some general DOM functions */
function TagName(node)
{
	if (!node) {return '';}

	if (ie)
	{
		if (!node.className) {return '';}
		return node.className.toLowerCase();
	}
	else
	{
		return node.nodeName.toLowerCase();
	}
}

function RemoveNode(x)
{
	if (!x) {return;}
	if (!x.parentNode) {return;}
	x.parentNode.removeChild(x);
}
function InsertBefore(n1, n2) //put n1 before n2
{
	if (!n1 || !n2) {return;}
	
	try
	{
		n2.parentNode.insertBefore(n1, n2);
	}
	catch(e)
	{
		return;
	}
}
function InsertAfter(n1, n2) //put n2 after n1
{
	if (!n1 || !n2) {return;}
	
	if (n1.nextSibling)
	{
		InsertBefore(n2, n1.nextSibling);
	}
	else
	{
		n1.parentNode.appendChild(n2);
	}
}
function PrependChild(parent, node) 
{
    if (parent.firstChild) 
	{
        parent.insertBefore(node, parent.firstChild);
    } 
	else 
	{
        parent.appendChild(node);
    }
}
function SwapNodes(n1, n2)
{
	if (!n1 || !n2) {return;}
	n1.parentNode.insertBefore(n2, n1);
}
function ChangeNodeName(node, new_name)
{
	var new_tag = document.createElement(new_name);
	InsertBefore(new_tag, node);

	while (node.firstChild)
	{
		new_tag.appendChild(node.firstChild);
	}
}
function ChangeNodeName2(node, new_name, class_name, at1, at2, at3, at4)
{
	var new_tag = document.createElement(new_name);
	if (class_name)
	{
		AddClassName(new_tag, class_name);
	}
	if (at1)
	{
		if (at1 != '') {new_tag.setAttribute(at1, at2);}
	}
	if (at3)
	{
		if (at3 != '') {new_tag.setAttribute(at3, at4);}
	}
	
	InsertBefore(new_tag, node);

	while (node.firstChild)
	{
		new_tag.appendChild(node.firstChild);
	}
}
function AddClassName(node, name)
{
	if (ie) {node.className = name;}
	else {node.setAttribute('class', name);}
}

function IsFirstChild(node)
{
	if (node == node.parentNode.firstChild) {return true;}
	return false;
}
function IsLastChild(node)
{
	if (node == node.parentNode.lastChild) {return true;}
	return false;
}
function IsOnlyChild(node)
{
	if (IsFirstChild(node) && IsLastChild(node)) {return true;}
	return false;
}
function Wrap(node, wrapper, class_name)
{
	var wrap_node = document.createElement(wrapper);
	if (class_name)
	{
		AddClassName(wrap_node, class_name);
	}
	InsertBefore(wrap_node, node);
	wrap_node.appendChild(node);
}

function InnerText(node)
{
	var fc = node.childNodes[0];
	if (!fc) {return '';}
	if (fc.nodeType != 3) {return '';}
	
	return fc.nodeValue;
}
function SetInnerText(node, str)
{
	var fc = node.childNodes[0];
	if (!fc) {return;}
	if (fc.nodeType != 3) {return;}
	
	fc.nodeValue = str;
}
function RemoveInnerText(node)
{
	RemoveNode(node.firstChild);
}



/* These two functions take a hell of a lot of time processing. We try and avoid them whenever possible*/
function OffsetHeight(node)
{
	return node.offsetHeight;
}
function OffsetWidth(node)
{
	return node.offsetWidth;
}

//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
var getElementsByClassName = function (className, tag, elm){
	if (document.getElementsByClassName) {
		getElementsByClassName = function (className, tag, elm) {
			elm = elm || document;
			var elements = elm.getElementsByClassName(className),
				nodeName = (tag)? new RegExp("\\b" + tag + "\\b", "i") : null,
				returnElements = [],
				current;
			for(var i=0, il=elements.length; i<il; i+=1){
				current = elements[i];
				if(!nodeName || nodeName.test(current.nodeName)) {
					returnElements.push(current);
				}
			}
			return returnElements;
		};
	}
	else if (document.evaluate) {
		getElementsByClassName = function (className, tag, elm) {
			tag = tag || "*";
			elm = elm || document;
			var classes = className.split(" "),
				classesToCheck = "",
				xhtmlNamespace = "http://www.w3.org/1999/xhtml",
				namespaceResolver = (document.documentElement.namespaceURI === xhtmlNamespace)? xhtmlNamespace : null,
				returnElements = [],
				elements,
				node;
			for(var j=0, jl=classes.length; j<jl; j+=1){
				classesToCheck += "[contains(concat(' ', @class, ' '), ' " + classes[j] + " ')]";
			}
			try	{
				elements = document.evaluate(".//" + tag + classesToCheck, elm, namespaceResolver, 0, null);
			}
			catch (e) {
				elements = document.evaluate(".//" + tag + classesToCheck, elm, null, 0, null);
			}
			while ((node = elements.iterateNext())) {
				returnElements.push(node);
			}
			return returnElements;
		};
	}
	else {
		getElementsByClassName = function (className, tag, elm) {
			tag = tag || "*";
			elm = elm || document;
			var classes = className.split(" "),
				classesToCheck = [],
				elements = (tag === "*" && elm.all)? elm.all : elm.getElementsByTagName(tag),
				current,
				returnElements = [],
				match;
			for(var k=0, kl=classes.length; k<kl; k+=1){
				classesToCheck.push(new RegExp("(^|\\s)" + classes[k] + "(\\s|$)"));
			}
			for(var l=0, ll=elements.length; l<ll; l+=1){
				current = elements[l];
				match = false;
				for(var m=0, ml=classesToCheck.length; m<ml; m+=1){
					match = classesToCheck[m].test(current.className);
					if (!match) {
						break;
					}
				}
				if (match) {
					returnElements.push(current);
				}
			}
			return returnElements;
		};
	}
	return getElementsByClassName(className, tag, elm);
};
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@




function GetElementsByTagName(name, ancestor_node)
{
	if (ie) //For IE, we've already changed eg. <math> to <span class="math">
	{
		return getElementsByClassName(name, null, ancestor_node);
	}
	else 
	{
		return ancestor_node.getElementsByTagName(name);
	}
}


/*@@@@@@      FUNCTIONS THAT  TIDY THE MATH       @@@@@@@@*/

var g_doctype = '';
var strict_doctype = false;

function nsResolver(prefix) {  //TODO: the getElementsByClassName() function has a namespaceResolver method which might also work
  var ns = {  
    'x' : 'http://www.w3.org/1999/xhtml',  
    'm': 'http://www.w3.org/1998/Math/MathML'  
  };  
  return ns[prefix] || null;  
}  

function CleanWhitespace(ancestor_node) /*TODO: when browsers support the css3 property      white-space-collapse:discard;      we won't need this*/
{
	var i; var lenny = 0; var remove_array = new Array();
	var text_node; var str = ''; var orig_str = str;

	var xPathResult = document.evaluate('.//math//text()', ancestor_node, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
	lenny = xPathResult.snapshotLength;
		
	if (lenny <= 0 && g_doctype.indexOf('mathml') != -1)
	{
		xPathResult = document.evaluate('.//m:math//text()', ancestor_node, nsResolver, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null );
		lenny = xPathResult.snapshotLength;
	}
	if (lenny <= 0)
	{
		xPathResult = document.evaluate('.//x:math//text()', ancestor_node, nsResolver, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null );
		lenny = xPathResult.snapshotLength;
	}
	
	for (i = 0; i < lenny; i++) 
	{

		text_node = xPathResult.snapshotItem(i);

		str = text_node.nodeValue;
		orig_str = str;
		str = str.replace(/[\t\n\r ]+/g, " ");

		if (TagName(text_node.parentNode) == 'mtext' || str == ', ' || str == '. ' || str == '; ') 
		{	//preserve leading/trailing space in mtext elements as this makes something like <mtext>if </mtext><mi>x</mi><mo>=</mo><mn>1</mn> look right
			if (str.charAt(0) == " " && !text_node.previousSibling && text_node.parentNode)
			{
				if (text_node.parentNode.innerHTML) {
					text_node.parentNode.innerHTML = '&#160;' + LeftTrim(str);
				} else {
					SetInnerText(text_node.parentNode, '\u00A0' + LeftTrim(str));
				}
			}
			if (str.charAt(str.length - 1) == " " && !text_node.nextSibling && text_node.parentNode)
			{
				if (text_node.parentNode.innerHTML) {
					text_node.parentNode.innerHTML = RightTrim(str) + '&#160;';
				} else {
					SetInnerText(text_node.parentNode, RightTrim(str) + '\u00A0');
				}
			}	
		}
		else
		{
			if (str.charAt(0) == " " && !text_node.previousSibling) 			{str = LeftTrim(str);}
			if (str.charAt(str.length - 1) == " " && !text_node.nextSibling)    {str = RightTrim(str);}
			
			if (str == '' || str == ' ') 
			{
				remove_array.push(text_node);
			}
			else if (str != orig_str)    
			{
				text_node.nodeValue = str;
			}
		}
	}

	
	lenny = remove_array.length; 
	if (lenny > 0)
	{
		for (i = 0; i < lenny; i++) 
		{
			RemoveNode(remove_array[i]);
		}
	}
	
	if (debug_on) {SetTime('CleanWhitespace()')}
}

function SortOutSize(ancestor_node) //fontsize and mathsize attributes
{
	var i; var lenny; var node; var fs = '';
	var xPathResult = document.evaluate('//*[@fontsize] | //*[@mathsize]', ancestor_node, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
	lenny = xPathResult.snapshotLength;

	for (i = 0; i < lenny; i++) 
	{
		node = xPathResult.snapshotItem(i);
		
		fs = node.getAttribute('fontsize');
		if (!fs || fs == '') {fs = node.getAttribute('mathsize');}
		
		try{
			if (fs && fs != '') {node.style.fontSize = fs;}
		}catch(e){}
	}
	
	if (debug_on) {SetTime('SortOutSize()')}
}
function SortOutColor(ancestor_node) //color and mathcolor attributes
{
	var i; var lenny; var node; var col = '';
	var xPathResult = document.evaluate('//*[@color] | //*[@mathcolor]', ancestor_node, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
	lenny = xPathResult.snapshotLength;

	for (i = 0; i < lenny; i++) 
	{
		node = xPathResult.snapshotItem(i);
		
		col = node.getAttribute('color');
		if (!col || col == '') {col = node.getAttribute('mathcolor');}

		try{
			if (col && col != '') {node.style.color = col;}
		}catch(e){}
	}
	
	if (debug_on) {SetTime('SortOutColor()')}
}
function SortOutBackground(ancestor_node)
{
	var i; var lenny; var node; var col = '';
	var xPathResult = document.evaluate('//*[@background]', ancestor_node, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
	lenny = xPathResult.snapshotLength;

	for (i = 0; i < lenny; i++) 
	{
		node = xPathResult.snapshotItem(i);
		
		col = node.getAttribute('background');
		try{
			if (col && col != '') {node.style.backgroundColor = col;}
		}catch(e){}
	}
	
	if (debug_on) {SetTime('SortOutBackground()')}
}
function SortOutSpace(ancestor_node)
{
	var i; var lenny; var node; var col = ''; var width = ''; var height = ''; var depth = ''; var lspace = ''; var rspace = '';
	var j; var spaces;
	
	for (j = 0; j < 2; j++)
	{
	
		if (j == 0) {spaces = GetElementsByTagName('mspace', ancestor_node);}
		else {spaces = GetElementsByTagName('mpadded', ancestor_node);}
		
		lenny = spaces.length;
		
		for (i = 0; i < lenny; i++) 
		{
			node = spaces[i];
			
			try{
				width = node.getAttribute('width');
				if (width && width != '') {node.style.width = width;}
			}catch(e){}	
			
			try{
				height = node.getAttribute('height'); 
				if (height && height != '') {node.style.bottom = height;} 
			}catch(e){}	

			try{
				depth = node.getAttribute('depth');  
				if (depth && depth != '')  {node.style.top = depth;}
			}catch(e){}	

			try{
				lspace = node.getAttribute('lspace'); 
				if (lspace && lspace != '') {
					node.style.marginLeft = lspace;
					//This hack is needed to display negated arrows properly (where / symbol used with negative lspace)
					if (lspace.charAt(0) == '-') {node.style.paddingRight = lspace.substring(1);}
				}
			}catch(e){}	
			
			try{
				rspace = node.getAttribute('rspace');
				if (rspace && rspace != '') {node.style.marginRight = rspace;}
			}catch(e){}	
		}
	
	}
	
	if (debug_on) {SetTime('SortOutSpace()')}
}


var cache_widths = true; //This helps with msubsup stuff where we cache the widths of previously calculated elements
var sub_ih = [];
var sub_widths = [];

function AddCachedWidth(ih, wid)
{
	sub_ih.push(ih);
	sub_widths.push(wid);
}

function GetCachedWidth(ih)
{
	var i; var lenny = sub_ih.length;
	if (lenny <= 0) {return 0;}
	
	for (i = 0; i < lenny; i++)
	{
		if (ih == sub_ih[i]) {return sub_widths[i];}
	}
	return 0;
}

function SortOutMsubsup(ancestor_node)
{
	if (!config.flushed_msubsup) {return;}
	if (ie && !config.ie_flushed_msubsup) {return;}
	
	var i; var lenny; var wid = 0; var node; var str = ''; var ih = ''; var cw = 0;
	

	var msubsup_array = GetElementsByTagName('msubsup', ancestor_node);
	lenny = msubsup_array.length;

	for (i = 0; i < lenny; i++) 
	{
		node = msubsup_array[i];
		
		wid = 0; cw = 0;
		if (node.childNodes[1]) 
		{
			if (cache_widths && TagName(node.parentNode) == 'math')
			{
				try{
					ih = node.childNodes[1].innerHTML;
					wid = GetCachedWidth(ih);
				}catch(e){}
			}
			if (wid == 0)
			{
				wid = OffsetWidth(node.childNodes[1]);
				if (cache_widths)
				{
					AddCachedWidth(ih, wid);
				}
			}
		}
		wid = -wid;
		wid = wid + 1;
		str = wid.toString()+'px';
		
		try{
			if (node.childNodes[2]) {node.childNodes[2].style.left = str;}
		}catch(e){}
	}
	
	if (debug_on) {SetTime('SortOutMsubsup()')}
}

function SortOutHorizontalStretchy(ancestor_node) //TODO: could integrate this with SortOutOps()
{
	var i; var lenny; var par; var par_name = ''; var cha = ''; var stretchy_attr = ''; var stretchchar_attr = '';
	var wid = -1; var accent = ''; var max_size = ''; var min_size = '';
	var node; var str = ''; var inrow = false;
	
	var mo_s = GetElementsByTagName('mo', ancestor_node);
	lenny = mo_s.length;

	for (i = 0; i < lenny; i++) 
	{
		node = mo_s[i];
		if (!node) {continue;}
		
		cha = Trim(InnerText(node));
		
		par = node.parentNode;
		par_name = TagName(par);
		inrow = false;
		
		if (par_name == 'mrow')
		{
			inrow = true;
			par = par.parentNode;
			par_name = TagName(par);
			if (!IsOnlyChild(node)) {continue;}
		}
		
		if (par_name == 'munder' || par_name == 'mover' || par_name == 'munderover' || par_name == 'munder2' || par_name == 'mover2' || par_name == 'munderover2' || par_name == 'munderover3')
		{
			var src = StretchyCode(cha);
			if (src == '') {continue;}

			//get the stretchy attribute and check it's not false
			stretchy_attr = node.getAttribute('stretchy');
			if (stretchy_attr == 'false') {continue;}
			// the attribute might be called stretchchar according to some mathml specifications
			stretchchar_attr = node.getAttribute('stretchchar');
			if (stretchchar_attr == 'false') {continue;}
			
			if (!config.auto_horizontal_stretch)
			{
				if (!stretchy_attr && !stretchchar_attr) {continue;}
			}
			
			
			//won't stretch if maxsize is 1 either
			max_size = node.getAttribute('maxsize');
			min_size = node.getAttribute('minsize');
			
			if (max_size == '1' || max_size == '1em') {continue;}
			
			if (stretchy_attr == 'true' || stretchchar_attr == 'true')
			{}
			else //if parent has accent or accentunder attributes, we won't need to stretch
			{
				if (par.getAttribute('accent') == 'true' || par.getAttribute('accentunder') == 'true') {continue;}
			}
			
			var img = document.createElement('img');
			img.setAttribute('class', 'no_shadow');
			img.setAttribute('src', 'MathHTML/Stretchy/'+src);
			
			//img.style.width = '100%'; //Setting width to 100% was too big for webkit browsers, so explicity  getting the width of the parent is safer (but less pure)

			wid = 0;
			
			if (node.previousSibling && !opera) 
			{
				wid = OffsetWidth(node.previousSibling);
			}
			else 
			{
				if (inrow)
				{
					wid = OffsetWidth(node.parentNode.parentNode);
				}
				else 
				{
					wid = OffsetWidth(node.parentNode);
				}
			}
			
			//Compare width to any maxsize or minsize attributes
			if (max_size && max_size != '')
			{
				var max_width = parseInt(max_size) * 16; 
				if (wid > max_width) {wid = max_width;} //TODO: calculate pixels per em. 16 is just a rough guess
			}
			
			if (min_size && min_size != '')
			{
				var min_width = parseInt(min_size) * 16; 
				if (wid < min_width) {wid = min_width;} //TODO: calculate pixels per em. 16 is just a rough guess
			}
			
			if (wid < 26) {continue;} //no point in stretching an image if it isn't that wide
			if (ie7 && wid < 35) {continue;}
			if (opera && wid < 35) {continue;}
			
			img.setAttribute('height', 12);
			
			if (ie7) {wid -= 10;}
			else if (ie) {wid -= 6;}
			
			img.setAttribute('width', wid);
			
			node.appendChild(img);
			RemoveNode(node.firstChild);	
		}
	}
	
	if (debug_on) {SetTime('SortOutHorizontalStretchy()')}
}

function ConvertAccents(ancestor_node) //Converts something like    <mover accent="true"><mi>x</mi><mo>^</mo></mover>    to     <mhat><mi>x</mi></mhat>
{
	if (!browser_supports_border_image) {return;}

	var i; var lenny; var cha = ''; var tag_name = ''; var accent_node; var accent; var src = ''; var node; var str = '';
	var at1 = ''; var at2 = '';	var at3 = ''; var at4 = '';	
	

	var movers = GetElementsByTagName('mover', ancestor_node);
	lenny = movers.length;

	for (i = 0; i < lenny; i++) 
	{
		node = movers[i];
		if (node.getAttribute('accent') != "true") {continue;}
	
		if (!node.firstChild) {continue;}
	
		if (node.childNodes[1])
		{
			accent_node = node.childNodes[1];
			if (TagName(accent_node) == 'mrow')
			{
				accent = InnerText(accent_node.firstChild);
			}
			else
			{
				accent = InnerText(accent_node);
			}
			src = AccentCode(accent);
			if (src == '') {continue;}
			
			at1 = ''; at2 = ''; at3 = ''; at4 = '';
			tag_name = TagName(node.firstChild);
			if (tag_name == 'mrow') 
			{
				cha = InnerText(node.firstChild.firstChild);
			}
			else 
			{
				cha = InnerText(node.firstChild);
			}
			if (!IsTallLetter(cha)) 
			{
				at1 = 'smallletter'; at2 = 'true';
			}
			/*if (IsSkinnyLetter(cha))
			{
				at3 = 'skinnyletter'; at4 = 'true';
			}*/
			
			RemoveNode(accent_node);
			ChangeNodeName2(node, 'm'+src, '', at1, at2, at3, at4);
		}
	}
	
	
	var munders = GetElementsByTagName('munder', ancestor_node);
	lenny = munders.length;
	
	for (i = 0; i < lenny; i++) 
	{
		node = munders[i];
		if (node.getAttribute('accent') != "true" && node.getAttribute('accentunder') != "true") {continue;}
	
		if (!node.firstChild) {continue;}
	
		if (node.firstChild.nextSibling)
		{
			accent_node = node.firstChild.nextSibling;
			if (TagName(accent_node) == 'mrow')
			{
				accent = InnerText(accent_node.firstChild);
			}
			else
			{
				accent = InnerText(accent_node);
			}
			
			src = AccentUnderCode(accent);
			if (src == '') {continue;}

			at1 = ''; at2 = '';
			tag_name = TagName(node.firstChild);
			if (tag_name == 'mrow') 
			{
				cha = InnerText(node.firstChild.firstChild);
			}
			else 
			{
				cha = InnerText(node.firstChild);
			}
			/*if (IsSkinnyLetter(cha))
			{
				at1 = 'skinnyletter'; at2 = 'true';
			}*/
			
			
			RemoveNode(accent_node);
			ChangeNodeName2(node, 'm'+src, '', at1, at2);
		}
	}
	
	if (debug_on) {SetTime('ConvertAccents()')}
}

function SortOutAccents(ancestor_node) //only relevant when browser_supports_border_image = false  (or when the accent is not standard)
{
	var i; var lenny; var cha = ''; var tag_name = ''; var accent; var first_node; var node; var str = '';
	
	var movers = GetElementsByTagName('mover', ancestor_node);
	lenny = movers.length;

	for (i = 0; i < lenny; i++) 
	{
		node = movers[i];
		if (node.getAttribute('accent') != "true") {continue;}

		try
		{
			//if what's under the accent is a small letter, then give a negative margin to make things look better
			tag_name = TagName(node.firstChild);
			if (tag_name == 'mrow') {first_node = node.firstChild.firstChild;}
			else {first_node = node.firstChild;}

			cha = InnerText(first_node);
			
			node.firstChild.style.lineHeight = '0.6em';
			node.firstChild.nextSibling.style.marginBottom = '-0.3em';
			
			if (!IsTallLetter(cha)) 
			{
				node.firstChild.nextSibling.style.lineHeight = '0.7em';
			}
			else  //accent looks better if we shift it to the right a bit. BUT it depends really on whether the letter is upright or italic
			{
				if (!ie) {node.firstChild.nextSibling.style.marginLeft = '0.15em';}
			}
			
			//give the accent no padding
			tag_name = TagName(node.firstChild.nextSibling);
			if (tag_name == 'mrow') {accent = node.firstChild.nextSibling.firstChild;}
			else {accent = node.firstChild.nextSibling}
			
			accent.style.paddingRight = '0em'; //Should be able to do this in the stylesheet instead
						
			if (accent.innerHTML == '.') {accent.innerHTML = '&#x2219;';}  //change dot to centered dot
			else if (accent.innerHTML == '..') {accent.innerHTML = '&#x2219;&#x2219;';}
			else if (accent.innerHTML == '\u2192') {accent.style.fontSize = '0.6em';} //stop right arrow being longer than the letter itself
		}
		catch(e)
		{}
	}
	
	if (debug_on) {SetTime('SortOutAccents()')}
}

function NoPaddingOnOp(cha)
{
	var lenny = nopadding_array.length;
	for (var i = 0; i < lenny; i++)
	{
		if (cha == nopadding_array[i]) {return true;}
	}
	return false;
}

function SortOutOps(ancestor_node)
{
	var i; var lenny; var node; var str = ''; var cha = ''; var largeop = ''; var padded_op = false;
	var form = ''; var fence = ''; var separator = ''; var lspace = ''; var rspace = '';
	var node2; var exit = false; var ds = ''; var ds2 = ''; var par; var par_name = '';
	var sl; var op_img = ''; var im; var char_code; var minus = false; var is_comma = false; var large = false; var rep_cha = '';

	var mo_s = GetElementsByTagName('mo', ancestor_node); 
	lenny = mo_s.length;

	for (i = 0; i < lenny; i++) 
	{
		node = mo_s[i];
		
		cha = InnerText(node);
		char_code = cha.charCodeAt(0);
		
		minus = false;
		if (cha == '-') //serif minus is too narrow, so in the css we use Lucida
		{
			node.setAttribute('lucida', 'true');
			minus = true;
		}
		//(small) spaces
		if (char_code >= 8192 && char_code <= 8203) {node.setAttribute('lucida', 'true');}
		//TODO: would it be better (and more reliable) if we gave any large char_code (eg > 256) a lucida attribute?
		
		
		if (cha == '.' || cha == '. ' || cha == ',' || cha == ', ')
		{
			//check if the previous sibling is a fraction. if so we will move the period or comma up a bit as this looks better
			if (node.previousSibling)
			{
				if (TagName(node.previousSibling) == 'mfrac')
				{
					try{
						node.style.position = 'relative';
						node.style.top = '-0.3ex';
					}catch(e){}
				}
			}
		}
		
		is_comma = false;
		if (cha == ',' || cha == ';' || cha == ', ' || cha == '; ' || cha == ',\u00A0' || cha == ';\u00A0')
		{
			is_comma = true;
			node.setAttribute('comma', 'true');
		}
		
		large = false;
		largeop = node.getAttribute('largeop');
		if (largeop == 'true') 
		{
			large = true;
		}
		
		sl = node.getAttribute('scriptlevel');
		if (sl && sl != '') {continue;}
		
	
		if (char_code > 256 && largeop != 'false' && !large) //certain operators will always be displayed large by default
		{
			if (cha == '\u2211') 	  {large = true;}	//summation
			else if (cha == '\u222B') {large = true;}	//integral
			else if (cha == '\u220F') {large = true;}	//product
			else if (cha == '\u2210') {large = true;}	//coproduct
			else if (IsIntegral(cha)) {large = true;}
		}
		
		
		par = node.parentNode;
		par_name = TagName(par);
		if (par_name == 'mrow')
		{
			par = par.parentNode;
			par_name = TagName(par);
		}
		
		if ((par_name == 'msub' || par_name == 'msup' || par_name == 'msubsup') && !large)
		{
			if (!NoPaddingOnOp(cha))
			{
				if (IsFirstChild(node))
				{
					//This gives good looking padding on something like   +_n   or   =_n
					try{
						par.style.paddingLeft = '0.25em';
						par.style.paddingRight = '0.25em';
					}catch(e){}
				}
			}
		}

		if (large)
		{
			if (IsIntegral(cha))
			{
				node.setAttribute('integral', 'true');
				if (par_name == 'msub' || par_name == 'msubsup') //subscript needs to be moved back to make things look good
				{
					try{
						//if (par.childNodes[1]) {par.childNodes[1].setAttribute('integralsubscript', 'true');}
						if (par.childNodes[1])
						{
							par.childNodes[1].style.position = 'relative';
							par.childNodes[1].style.left = '-0.5em';
						}
						
					}catch(e){}
				}
			}
			
			
			op_img = '';
			if (config.use_images_for_large_operators)
			{
				op_img = LargeOperatorCode(cha);
				if (op_img != '') {op_img = 'MathHTML/LargeOperators/'+op_img;}
				
				RemoveNode(node.firstChild);
				
				im = document.createElement('img');
				im.setAttribute('class', 'no_shadow');
				im.setAttribute('src', op_img);
				//im.style.display = 'block'; //makes things worse;
				
				//prevent image from being dragged
				im.setAttribute('onmousedown', 'if (event.preventDefault) event.preventDefault()');
				if (largeop != 'true') {node.setAttribute('largeop', 'true');}

				node.appendChild(im);
				
				if (par_name == 'munder' || par_name == 'mover' || par_name == 'munderover')
				{
					par.setAttribute('haslargeop', 'true');
					
					if (firefox && strict_doctype && par_name == 'munderover' && par.childNodes[2] && TagName(par.childNodes[2]) == 'mrow')
					{
						par.childNodes[2].style.marginBottom = '-0.3ex';
					}
				}
				else if (par_name == 'msubsup' && !strict_doctype)
				{
					if (par.childNodes[2].style) {par.childNodes[2].style.top = '-0.46em';}
				}
				else if (par_name == 'msup' && !strict_doctype)
				{
					if (par.childNodes[1].style) {par.childNodes[1].style.top = '-0.46em';}
				}
			}
			else
			{
				if (largeop != 'true') {node.setAttribute('largeop', 'true');}
				node.setAttribute('usingfonts', 'true');
			}
		}
		else //Convert characters which are unrecognized by all browsers on a 'default' operating system with no math fonts installed.
		{
			rep_cha = CharacterReplacement(cha);
			if (rep_cha != '')
			{
				try{
					node.innerHTML = rep_cha;
				}catch(e){
					try{SetInnerText(node, rep_cha);}catch(e){}
				}
			}
		}

		//Find operators which should be padded
		padded_op = false;
		form = node.getAttribute('form'); fence = node.getAttribute('fence'); separator = node.getAttribute('separator')
		
		if (fence == 'true' || separator == 'true') // css handles these
		{}
		else if (form == 'infix' || form == 'prefix' || form == 'postfix') // css handles these
		{}
		else if (node.getAttribute('padded') == 'false') // css handles these
		{}
		else if (is_comma) // css handles these
		{}
		else if (large)
		{}
		else
		{		
			if (IsFirstChild(node)) 
			{
				if (par_name == 'mtd') //for aligned equations, we want padding on the op even though it's the first child
				{
					if (par.getAttribute('columnalign') == 'left') 
					{
						padded_op = true;
					}
					else {}
				}
				else {}
			}
			else if (IsLastChild(node)) 
			{}
			else if (NoPaddingOnOp(cha)) 
			{}
			else if (minus) //Need to figure out if it's the negation symbol or not. Do this by looking at either sibling 
			{
				if (node.nextSibling && node.previousSibling)
				{
					if (TagName(node.nextSibling) == 'mo' || TagName(node.previousSibling) == 'mo')
					{}
					else {padded_op = true;}
				}
				else {padded_op = true;}
			}
			else
			{
				padded_op = true;
			}	
		}

		if (padded_op)
		{
			node.setAttribute('form', 'infix');
		}
		

		//Of course, if we have lspace or rspace attributes, these will override the above
		lspace = node.getAttribute('lspace');
		rspace = node.getAttribute('rspace');
		
		try{
			if (lspace && lspace != '') {node.style.paddingLeft = lspace;}
			if (rspace && rspace != '') {node.style.paddingRight = rspace;}
		}catch(e){}
	}
	
	if (debug_on) {SetTime('SortOutOps()')}
}


//For browsers which don't support border-image, replace     <mfrenced open="(" close=")"> ... <mfenced>     by     <mrow><mo stretchy="true">(</mo>..........<mo stretchy="true">)</mo></mrow>
function SortOutFenced(ancestor_node) 
{
	var i; var lenny; var node; var str = ''; var open = ''; var close = '';
	
	var fenced = GetElementsByTagName('mfenced', ancestor_node);
	lenny = fenced.length;

	for (i = 0; i < lenny; i++) 
	{
		node = fenced[i];
		
		open = node.getAttribute('open');
		close = node.getAttribute('close');
		
		if (browser_supports_border_image)
		{
			if (!open && !close) {continue;}
			if (open == '(' && close == ')') {continue;}
			if (open == '[' && close == ']') {continue;}
			if (open == '{' && close == '}') {continue;}
			if (open == '{' && close == '')  {continue;}
			if (open == '' && close == '}')  {continue;}
			if (open == '\u2229' && close == '\u222A') {continue;} //angular
			if (open == '\u27E8' && close == '\u27E9') {continue;} //angular
			if (open == '\u230A' && close == '\u230B') {continue;} //floor
			if (open == '\u2308' && close == '\u2309') {continue;} //ceiling
			if (open == '[' && close == ')') {continue;}
			if (open == '(' && close == ']') {continue;}
			
			if (open == '' && close == ']') {continue;}
			if (open == '[' && close == '') {continue;}
			if (open == '/' && close == '') {continue;}
			if (open == '\\' && close == '') {continue;}
			if (open == '' && close == '/') {continue;}
			if (open == '' && close == '\\') {continue;}
		}
		
		if (!ie7)
		{
			// for 'absolute value' or 'norm', we just use border: 1px solid   or   4px double        in the css
			if ((open == '|' || open == '\u2223' || open == '||' || open == '\u2225' || open == '') && (close == '|' || close == '\u2223' || close == '||' || close == '\u2225' || close == ''))
			{
				continue;
			}
		}
		
		var open_node; var close_node;
		if (ie)
		{
			open_node = document.createElement('span');  open_node.className = 'mo';
			close_node = document.createElement('span'); close_node.className = 'mo';		
		}
		else
		{
			open_node = document.createElement('mo');
			close_node = document.createElement('mo');
		}
		
		open_node.setAttribute('stretchy', 'true'); close_node.setAttribute('stretchy', 'true');
				
		try
		{
			open_node.innerHTML = open; close_node.innerHTML = close;
		}
		catch(e)
		{
			SetInnerText(open_node, open); SetInnerText(close_node, close);
		}
		
		PrependChild(node, open_node);
		node.appendChild(close_node);
		
		ChangeNodeName(node, 'mrow');
	}
	
	if (debug_on) {SetTime('SortOutFenced()')}
}

function IsBigNode(node)
{
	var tn = TagName(node);
	
	//|| tn == 'msub' || tn == 'msup' || tn == 'msubsup'    (decided things are more likely to work better without these)
	if (tn == 'mfrac' || tn == 'munder' || tn == 'mover' || tn == 'munderover' || tn == 'mrow' || tn == 'mtable' || tn == 'mfenced' || tn == 'msqrt' || tn == 'mroot' || tn == 'menclose' || tn == 'ie7table' || tn == 'ncr') {return true;}
	
	return false;
}

function SortOutParens(ancestor_node) //In general, this is only invoked when      browser_supports_border_image = false
{
	var i; var lenny; var node; var str = ''; var cha = ''; var stretch = false; 
	var stretchy_attr = ''; var stretchchar_attr = ''; var fence_attr = '';
	var src = ''; var max_size = ''; var min_size = '';
	var ht = 0; var ht2 = 0; var last_ht = 0; var diff = 1000;
	var match = ''; var test_node; var score = 0; var safe = 0; var getout = false;

	var mo_s = GetElementsByTagName('mo', ancestor_node); 
	lenny = mo_s.length;

	var large = false;

	for (i = 0; i < lenny; i++) 
	{
		node = mo_s[i];
		
		cha = Trim(InnerText(node));
		
		stretch = false; src = '';
		
		if (node.getAttribute('ignore') == 'true') {continue;}

		match = '';
		
		if (cha == '(') 		{match = ')';}
		else if (cha == '[') 	{match = ']';}
		else if (cha == '{') 	{match = '}';}
		else if (cha == '|') 	{match = '|';}
		else if (cha == '||') 	{match = '||';}
		else if (cha == '\u2223') 	{match = '\u2223';}
		else if (cha == '\u2225') 	{match = '\u2225';}
		else if (cha == '\u2329') 	{match = '\u232A';}
		else if (cha == '\u27E8') 	{match = '\u27E9';}
		else if (cha == '\u230A') 	{match = '\u230B';}
		else if (cha == '\u2308') 	{match = '\u2309';}

		src = ParenthesisCode(cha);
		if (src == '') {continue;}
		
		max_size = node.getAttribute('maxsize');
		min_size = node.getAttribute('minsize');
		stretchy_attr = node.getAttribute('stretchy');
		stretchchar_attr = node.getAttribute('stretchchar');
		fence_attr = node.getAttribute('fence');
		
		if (!config.stretch_single_parentheses)
		{
			if (stretchy_attr == 'true' || stretchchar_attr == 'true' || fence_attr == 'true')
			{}
			else {continue;}
		}
		if (stretchchar_attr == 'false') {continue;}
		if (fence_attr == 'false') {continue;}
		
		if (stretchy_attr == 'true') {stretch = true;}
		else if (stretchy_attr == 'false') {stretch = false;}
		else if (src != '' && max_size != '1' && max_size != '1em' && max_size != '1ex') {stretch = true;}
		else {stretch = false;}

		
		if (stretch)
		{
			if (match != '')
			{
				score = 1; safe = 0; getout = false; test_node = node.nextSibling;
				while (test_node && safe < 100)
				{
					if (IsBigNode(test_node)) {break;}
					if (TagName(test_node) == 'mo')
					{
						if (InnerText(test_node) == match)
						{
							score--;
							if (score == 0) {getout = true; test_node.setAttribute('ignore', 'true'); break;}
						}
						else if (InnerText(test_node) == cha) {score++;}
					}
					test_node = test_node.nextSibling; safe++;
				}
				if (getout) {continue;}
			}
		
			var img = document.createElement('img');
			img.setAttribute('class', 'no_shadow');
			img.setAttribute('src', 'MathHTML/Parentheses/'+src);
			//prevent image from being dragged
			img.setAttribute('onmousedown', 'if (event.preventDefault) event.preventDefault()');
			
			ht = OffsetHeight(node.parentNode);//This takes a lot of time!
			
			if (src == 'pipe.png') 
			{
				if (ie) 
				{
					if (ie7) {img.style.width = '8px';} else {img.style.width = '2px';}
				}
				else {img.style.width = '1px';} //looked a bit scrwny on IE so gave it 2px width
			}
			else 
			{
				if (ht > 60)
				{
					img.style.width = '0.55em';
				}
				else if (ht > 40)
				{	
					img.style.width = '0.50em';
				}
				else {img.style.width = '0.45em';}
			}

			if (max_size && max_size != '')
			{
				var max_height = parseInt(max_size) * 16; 
				if (ht > max_height) {ht = max_height;} //TODO: calculate pixels per em. 16 is just a rough guess
			}
			
			if (min_size && min_size != '')
			{
				var min_height = parseInt(min_size) * 16; 
				if (ht < min_height) {ht = min_height;} //TODO: calculate pixels per em. 16 is just a rough guess
			}
			
			diff = ht - last_ht;   //Have problems with matching parens being a different size when in msqrt so this is a hack to sort things out 
			if (diff <= 7 && diff >= -7) {ht = last_ht;}
			if (ht <= 22) {continue;} //normal font brackets should look fine in this case
			
			if (ie7)
			{
				if (ht < 40) {continue;}
				if (ht <= 50) {node.style.fontSize = '1.14em'; continue;}
			}
			else
			{
				if (ht <= 30) 
				{
					try{
						node.style.fontSize = '1.14em';
					}catch(e){} 
					
					continue;
				}
			}
			
			if (ie7) {real_ht = ht - 12;}
			else 
			{
				real_ht = ht - 6;
			} 
			
			img.style.height = real_ht+'px';
			//img.style.display = 'block'; //made things worse
			
			if (config.use_images_for_large_unfenced_parentheses)
			{
				RemoveInnerText(node);
				
				node.setAttribute('verticalalign', 'middle');
				img.setAttribute('verticalalign', 'middle');
				try{
					node.style.verticalAlign = 'middle';
				}catch(e){}
				
				node.appendChild(img);
			}
			else 
			{
				ht2 = ht - 5;
				
				node.setAttribute('verticalalign', 'middle');
				try{
					node.style.verticalAlign = 'middle';
				}catch(e){}
				
				node.setAttribute('squashfont', 'true');
				node.style.fontSize = ht2.toString() + 'px';
			}
			
			last_ht = ht;
		}
	}
	
	if (debug_on) {SetTime('SortOutParens()')}
}

function SortOutMI(ancestor_node) //rather cosmetic. Ensures that letter f is given enough padding, and in something like <mi>sin</mi>, the sin appears upright
{
	var i; var lenny; var node; var str = ''; var cha = ''; var ptn = '';
		
	var mi_s = GetElementsByTagName('mi', ancestor_node);
	lenny = mi_s.length;

	for (i = 0; i < lenny; i++) 
	{
		node = mi_s[i];
		cha = Trim(InnerText(node));
		
		if (cha == 'f') 
		{
			node.setAttribute('char', 'f');
		}

		if (cha.length > 1) 
		{
			node.setAttribute('word', 'true');
		}
	}
	
	if (debug_on) {SetTime('SortOutMI()')}
}


function AppleWrap(ancestor_node) //For webkit browsers, the six tags below need their children wrapped in an mrow node for things to display properly
{       						 //UPDATE: chrome doesn't seem to need this anymore but safari does
								//UPDATE2: safari doesn't seem to need this so maybe I changed something in the css which made it unnecessary
	var i; var lenny; var node; var str = '';
	
	var xPathResult = document.evaluate('//mover | //munder | //munderover | //msub | //msup | //msubsup', ancestor_node, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
	lenny = xPathResult.snapshotLength;
	
	for (i = 0; i < lenny; i++) 
	{
		node = xPathResult.snapshotItem(i);
		
		if (node.firstChild && TagName(node.firstChild) != 'mrow')
		{
			Wrap(node.firstChild, 'mrow');
		}
		if (node.childNodes[1] && TagName(node.childNodes[1]) != 'mrow')
		{
			Wrap(node.childNodes[1], 'mrow');
		}
		if (node.childNodes[2] && TagName(node.childNodes[2]) != 'mrow')
		{
			Wrap(node.childNodes[2], 'mrow');
		}
	}
	
	if (debug_on) {SetTime('AppleWrap()')}
}
function AppleUnderOver(ancestor_node) //Apple webkit browsers don't get the vertical align right on tables so we have a hack in the css which corrects this
{
	var i; var lenny; var node; var str = ''; var nodes; var nodes2; var nodes3;

	nodes = GetElementsByTagName('munderover', ancestor_node);
	lenny = nodes.length;

	for (i = 0; i < lenny; i++) 
	{
		node = nodes[i];
		node.setAttribute('apple', 'true');
	}
	
	nodes2 = GetElementsByTagName('mover', ancestor_node);
	lenny = nodes2.length;

	for (i = 0; i < lenny; i++) 
	{
		node = nodes2[i];
		node.setAttribute('apple', 'true');
	}
	
	nodes3 = GetElementsByTagName('munder', ancestor_node);
	lenny = nodes3.length;

	for (i = 0; i < lenny; i++) 
	{
		node = nodes3[i];
		node.setAttribute('apple', 'true');
	}
	
	if (debug_on) {SetTime('AppleUnderOver()')}
}


function ConvertForIE(node) //Since IE doesn't recognise the math tags directly, we need to grab the innerHTML and change e.g. <math> to <span class="math">
{
	var ih = node.innerHTML; 
	
	//Get rid of whitespace in math nodes. 
	ih = ih.replace(/>[\t\n\r ]+<math/gi, '>&#160;<math'); //want to preserve any space which occurs before a math element
	ih = ih.replace(/ <\/mtext/gi, '&#160;</mtext');
	ih = ih.replace(/<mtext> /gi, '<mtext>&#160;');
	ih = ih.replace(/>, <m/gi, '>,&#160;<m');
	ih = ih.replace(/>; <m/gi, '>;&#160;<m');
	ih = ih.replace(/>\. <m/gi, '>.&#160;<m');
	ih = ih.replace(/>[\t\n\r ]+<m/gi, '><m'); //Remove any space between math tags. The 'm' in here ensures it's only mathML tags which are affected 
											   //(unless it's a standard html tag which starts with m, but thre aren't many of those (ie map, menu, meta are not important ))
	
	//TODO: if we have space in the math tags like  <mi> x </mi> we'll want to remove that. Test the two below. //ACTUALLY, i've tested this and it doesn't seem to be needed
	//ih = ih.replace(/[\t\n\r ]+<\/m/gi, '</m');
	//ih = ih.replace(/<m([^>])*>[\t\n\r ]+/gi, '<m$1>');
	
	
	var math_attr = '';
	if (config.show_mathml_on_mousedown)
	{
		math_attr = ' onmousedown="mathml.ShowInnerMathHTML(this)"';
	}
	else if (config.magnify_on_mousedown)
	{
		math_attr = ' onmousedown="mathml.ShowBigSize(this)" onmouseup="mathml.RestoreSize(this)"';
	}
	
	ih = ih.replace(/<math/gi, '<span class="math"'+math_attr); ih = ih.replace(/<\/math>/gi, '</span>');
	
	//Note that for the below to work correctly, we have to have the longest words first (eg <msup must come before <ms )	
	ih = ih.replace(/<mlabeledtr/gi, '<span class="mlabeledtr"'); ih = ih.replace(/<\/mlabeledtr>/gi, '</span>');
	ih = ih.replace(/<moverbrace/gi, '<span class="moverbrace"'); ih = ih.replace(/<\/moverbrace>/gi, '</span>');
	ih = ih.replace(/<munderbrace/gi, '<span class="munderbrace"'); ih = ih.replace(/<\/munderbrace>/gi, '</span>');	
	ih = ih.replace(/<munderover/gi, '<span class="munderover"'); ih = ih.replace(/<\/munderover>/gi, '</span>');
	ih = ih.replace(/<menclose/gi, '<span class="menclose"'); ih = ih.replace(/<\/menclose>/gi, '</span>');
	ih = ih.replace(/<mlongdiv/gi, '<span class="mlongdiv"'); ih = ih.replace(/<\/mlongdiv>/gi, '</span>');
	ih = ih.replace(/<mphantom/gi, '<span class="mphantom"'); ih = ih.replace(/<\/mphantom>/gi, '</span>');
	ih = ih.replace(/<mpadded/gi, '<span class="mpadded"'); ih = ih.replace(/<\/mpadded>/gi, '</span>');
	ih = ih.replace(/<msubsup/gi, '<span class="msubsup"'); ih = ih.replace(/<\/msubsup>/gi, '</span>');
	ih = ih.replace(/<mfenced/gi, '<span class="mfenced"'); ih = ih.replace(/<\/mfenced>/gi, '</span>');
	ih = ih.replace(/<mscarry/gi, '<span class="mscarry"'); ih = ih.replace(/<\/mscarry>/gi, '</span>');
	ih = ih.replace(/<maction/gi, '<span class="maction"'); ih = ih.replace(/<\/maction>/gi, '</span>');	
	ih = ih.replace(/<mstyle/gi, '<span class="mstyle"'); ih = ih.replace(/<\/mstyle>/gi, '</span>');
	ih = ih.replace(/<mspace/gi, '<span class="mspace"'); ih = ih.replace(/<\/mspace>/gi, '</span>');
	ih = ih.replace(/<mtable/gi, '<span class="mtable"'); ih = ih.replace(/<\/mtable>/gi, '</span>');
	ih = ih.replace(/<mstack/gi, '<span class="mstack"'); ih = ih.replace(/<\/mstack>/gi, '</span>');
	ih = ih.replace(/<msline/gi, '<span class="msline"'); ih = ih.replace(/<\/msline>/gi, '</span>');
	ih = ih.replace(/<munder/gi, '<span class="munder"'); ih = ih.replace(/<\/munder>/gi, '</span>');
	ih = ih.replace(/<mover/gi, '<span class="mover"'); ih = ih.replace(/<\/mover>/gi, '</span>');
	ih = ih.replace(/<mtext/gi, '<span class="mtext"'); ih = ih.replace(/<\/mtext>/gi, '</span>');
	ih = ih.replace(/<msqrt/gi, '<span class="ie_msqrt"'); ih = ih.replace(/<\/msqrt>/gi, '</span>');
	ih = ih.replace(/<mroot/gi, '<span class="ie_mroot"'); ih = ih.replace(/<\/mroot>/gi, '</span>');
	ih = ih.replace(/<mfrac/gi, '<span class="mfrac"'); ih = ih.replace(/<\/mfrac>/gi, '</span>');
	ih = ih.replace(/<mrow/gi, '<span class="mrow"'); ih = ih.replace(/<\/mrow>/gi, '</span>');
	ih = ih.replace(/<msub/gi, '<span class="msub"'); ih = ih.replace(/<\/msub>/gi, '</span>');
	ih = ih.replace(/<msup/gi, '<span class="msup"'); ih = ih.replace(/<\/msup>/gi, '</span>');
	ih = ih.replace(/<mtr/gi, '<span class="mtr"'); ih = ih.replace(/<\/mtr>/gi, '</span>');
	ih = ih.replace(/<mtd/gi, '<span class="mtd"'); ih = ih.replace(/<\/mtd>/gi, '</span>');
	ih = ih.replace(/<mo/gi, '<span class="mo"'); ih = ih.replace(/<\/mo>/gi, '</span>');
	ih = ih.replace(/<mi/gi, '<span class="mi"'); ih = ih.replace(/<\/mi>/gi, '</span>');
	ih = ih.replace(/<mn/gi, '<span class="mn"'); ih = ih.replace(/<\/mn>/gi, '</span>');
	ih = ih.replace(/<ms/gi, '<span class="ms"'); ih = ih.replace(/<\/ms>/gi, '</span>');
	
	ih = ih.replace(/<none/gi, '<span class="none"'); ih = ih.replace(/<\/none>/gi, '</span>');
	ih = ih.replace(/<annotation/gi, '<span class="annotation"'); ih = ih.replace(/<\/annotation>/gi, '</span>');
	ih = ih.replace(/<annotation-xml/gi, '<span class="annotation-xml"'); ih = ih.replace(/<\/annotation-xml>/gi, '</span>');
	
	ih = ih.replace(/ fontsize="/gi, ' style="font-size:');
	ih = ih.replace(/ mathsize="/gi, ' style="font-size:');
	ih = ih.replace(/ mathcolor="/gi, ' style="color:');
	ih = ih.replace(/ color="/gi, ' style="color:');
	ih = ih.replace(/ background="/gi, ' style="background-color:');
	ih = ih.replace(/ width="/gi, ' style="width:');
	ih = ih.replace(/ height="/gi, ' style="padding-top:');
	ih = ih.replace(/ depth="/gi, ' style="padding-bottom:');
	ih = ih.replace(/ lspace="/gi, ' style="margin-left:');
	ih = ih.replace(/ rspace="/gi, ' style="margin-right:');	
	
	node.innerHTML = ih;
	
	if (debug_on) {SetTime('ConvertForIE()')}
}



function ShowBigSize(t)
{	
	t.setAttribute('magnify', 'true');
}
function RestoreSize(t)
{
	t.setAttribute('magnify', 'false');
}
function SetSizeChangeEvents(ancestor_node)
{
	var node; var i; var lenny;

	var math_tags = GetElementsByTagName('math', ancestor_node);
	
	lenny = math_tags.length;

	for (i = 0; i < lenny; i++) 
	{
		node = math_tags[i];
		node.setAttribute('onmousedown', 'mathml.ShowBigSize(this)');
		node.setAttribute('onmouseup', 'mathml.RestoreSize(this)');
		node.setAttribute('cursor', 'pointer');
	}
}
function ShowInnerMathHTML(t) //The original mathml is stored in a comment node after the math node
{							  //TODO: IE still screws this up by registering a mousedown event even when a propmt is already open
	if (!t.nextSibling) {return false;}
	if (t.nextSibling.nodeType != 8) {return false;}
	
	prompt('', t.nextSibling.nodeValue);
	return false;
}
function SetMouseDownEvents(ancestor_node)
{
	var node; var i; var lenny; var inline; var comment; var ih; var safe; var p1; var p2;

	var math_tags = GetElementsByTagName('math', ancestor_node);
	
	lenny = math_tags.length;

	for (i = 0; i < lenny; i++) 
	{
		node = math_tags[i];
		inline = true;
		if (node.getAttribute('display') == 'block') {inline = false;}
		
		if (!ie) {
			node.setAttribute('onmousedown', 'mathml.ShowInnerMathHTML(this)');
		}
		
		ih = node.innerHTML;
		
		safe = 0;
		while (ih.indexOf('<!--') != -1 && safe < 1000)
		{
			p1 = ih.indexOf('<!--');
			p2 = ih.indexOf('-->'); if (p2 == -1) {p2 = 30000;}
			
			ih = ih.substring(0, p1) + ih.substring(p2 + 3);
			safe++;
		}
		
		if (inline){
			comment = document.createComment('<math>' + ih + '</math>');
		}else{
			comment = document.createComment('<math display="block">' + ih + '</math>');
		}
		
		InsertAfter(node, comment);
	}
}



function SortOutRoots(ancestor_node) //This is for when the browser doesn't support border image //TODO: don't forget <menclose notation="radical">
{
	var i; var lenny; var node;

	var sqrts = GetElementsByTagName('msqrt', ancestor_node);
	lenny = sqrts.length;

	for (i = 0; i < lenny; i++) 
	{
		node = sqrts[i];
		node.setAttribute('noborderimage', 'true');
	}
	
	var roots = GetElementsByTagName('mroot', ancestor_node);
	lenny = roots.length;

	for (i = 0; i < lenny; i++) 
	{
		node = roots[i];
		node.setAttribute('noborderimage', 'true');
	}
	
	if (debug_on) {SetTime('SortOutRoots()')}
}

function GetTop(node)
{
	return node.offsetTop;
}
function FractionTest() //returns true if the fractions are in proper alignment (ie. browser is in standards mode (we have a strict doctype))
{
	try
	{
		var test_node = document.createElement('span');

		test_node.setAttribute('id', 'fractiontestnode');
		
		var ih = '<math display="block"><mi id="thisisQ">Q</mi><mfrac id="thisisS" style="vertical-align:-1.3ex;"><mi>S</mi><mi>T</mi></mfrac></math>';
		
		test_node.innerHTML = ih; //TODO: this might fail for xhtml pages. If the doctype is xhtml transitional then quirky fractions might still get displayed
		
		var bd = document.getElementsByTagName('body')[0];
		//TODO: EVAN TEST THIS!
		//PrependChild(bd, test_node);
		bd.appendChild(test_node);
		var q_top = GetTop(D('thisisQ'));
		var s_top = GetTop(D('thisisS'));
		
		//if position of S is lower than Q then we have a quirky fraction
		if (s_top > q_top) 
		{
			RemoveNode(D('fractiontestnode'));
			return false;
		}

		RemoveNode(D('fractiontestnode'));
		return true;
	}
	catch(e)
	{
		return true;
	}
}
var standard_fractions = false;
var done_fraction_test = false;
function QuirkyFractions(ancestor_node)
{
	if (strict_doctype || standard_fractions) {return;}
	
	var dt = g_doctype;
	
	if (dt != '')
	{
		if (dt.indexOf('strict') != -1 || dt.indexOf('xhtml 1.1') != -1 || dt.indexOf('xhtml 2.0') != -1 || dt.indexOf('mathml') != -1)
		{
			//alert('sd')
			strict_doctype = true; standard_fractions = true; return;
		}
	}
	
	if (!done_fraction_test)
	{
		standard_fractions = FractionTest();
		done_fraction_test = true;
	}
		
	if (!standard_fractions)
	{
		var i; var lenny; var node;
		var fracs = GetElementsByTagName('mfrac', ancestor_node);
		lenny = fracs.length;

		for (i = 0; i < lenny; i++) 
		{
			node = fracs[i];
			node.setAttribute('quirkyfraction', 'true');
			//node.style.verticalAlign = '0.6ex';
			try{
				if (chrome) {node.innerHTML = node.innerHTML;} //fixes a strange bug in chrome where sometimes the fractions still didn't align
			}catch(e){}
			//if (chrome) {node.style.color = '#000100';} 
		}
	}
}


function GetDoctype()
{
	if (document.doctype)
	{
		var dd = "<!DOCTYPE " + document.doctype.name + " PUBLIC \"" + document.doctype.publicId + "\"" + (document.doctype.systemId ?  " \"" + document.doctype.systemId + "\"" : "") + ">";
		return dd.toLowerCase();
	}
	return '';
}
function GotMathPlayer() //only relvant for IE with a mathml doctype
{
	try 
	{
		var mp = new ActiveXObject("MathPlayer.Factory.1");
		return true;
	}
	catch(e)
	{
		return false;
	}
}
function AddStyle(href)
{
	var css_node = document.createElement('link');
	css_node.type = 'text/css';
	css_node.rel = 'stylesheet';
	css_node.href = href;
	document.getElementsByTagName('head')[0].appendChild(css_node);
}




function tidy(node)
{//debugger
	if (!node) {return;}
	if (debug_on) {SetTime('START');}
	
	if (ie) {ConvertForIE(node);}
	
	if (!ie) {QuirkyFractions(node);}
	
	if (!ie) {CleanWhitespace(node);}
		
	if (config.show_mathml_on_mousedown) {SetMouseDownEvents(node);}
	if (!ie) {	
		if (config.magnify_on_mousedown) {SetSizeChangeEvents(node);}
	}
	
	if (browser_supports_border_image)
	{
		ConvertAccents(node);
	}
	else
	{
		if (!ie) {SortOutRoots(node);}
	}
	
	SortOutMI(node);
	
	if (apple) 
	{		   
		//if (safari) {AppleWrap(node);}    //mover, munder, and munderover stuff sometimes doesn't display properly in webkit if any of the children isn't wrapped in an mrow node
		                    //ALSO some msub/msup stuff needs wrapping to display properly //UPDATE: might not be necessary anymore
		
		AppleUnderOver(node);  //Apple webkit browsers don't get the vertical align right on tables so we have a hack in the css which corrects this
	}
	
	SortOutFenced(node);
	
	SortOutOps(node);
	
	if (!ie) {SortOutSize(node);}
	if (!ie) {SortOutSpace(node);}
	
	SortOutParens(node);
	
	SortOutMsubsup(node);
	
	SortOutAccents(node);
	
	SortOutHorizontalStretchy(node);
		
	if (!ie) {SortOutColor(node);}
	if (!ie) {SortOutBackground(node);}
	
	try{
		node.style.visibility = 'visible';	//in case the visibility has been set to 'hidden' in the css.
	}catch(e){
		node.setAttribute('visible', 'true');
	}

	
	if (debug_on) {SetTime('FINISH'); alert(time_str);}
}


var css_loaded = false; //only relevant for pages with a mathml doctype where the stylesheet is loaded dynamically for browsers which don't support mathml

function init()
{//debugger

	g_doctype = GetDoctype();
	if (g_doctype.indexOf('mathml') != -1)
	{
		if (firefox || opera) {return;}
		if (ie && GotMathPlayer()) {return;}
		
		if (!css_loaded)
		{
			AddStyle('mathml.css');
			css_loaded = true;
		}
	}
	
	
	var bd = document.getElementsByTagName('body')[0]; if (!bd) {var tmr = setTimeout('init()', 100); return;}
	var mathml_classes = getElementsByClassName('mathml'); var l = mathml_classes.length;
	if (l > 0) {
		for (var i = 0; i < l; i++) {tidy(mathml_classes[i]);}
	} else {
		tidy(bd);
	}
	
}


try{
	if (config.tidy_on_load) {window.onload = init;}
}catch(e){
	window.onload = init;
}


//Make the following variables and functions public
return {
	tidy:tidy,
	init:init,
	config:config,
	
	ShowBigSize:ShowBigSize,
	RestoreSize:RestoreSize,
	ShowInnerMathHTML:ShowInnerMathHTML
};



}();
